@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.11
 #-------------------------------------------------------------------------------
 # scripts/readelf.py
 #
@@ -9,6 +9,7 @@
 #-------------------------------------------------------------------------------
 import argparse
 import os, sys
+import re
 import string
 import traceback
 import itertools
@@ -26,8 +27,7 @@ sys.path.insert(0, '.')
 
 from elftools import __version__
 from elftools.common.exceptions import ELFError
-from elftools.common.py3compat import (
-        ifilter, byte2int, bytes2str, itervalues, str2bytes, iterbytes)
+from elftools.common.utils import bytes2str, iterbytes
 from elftools.elf.elffile import ELFFile
 from elftools.elf.dynamic import DynamicSection, DynamicSegment
 from elftools.elf.enums import ENUM_D_TAG
@@ -48,7 +48,7 @@ from elftools.elf.descriptions import (
     describe_symbol_type, describe_symbol_bind, describe_symbol_visibility,
     describe_symbol_shndx, describe_reloc_type, describe_dyn_tag,
     describe_dt_flags, describe_dt_flags_1, describe_ver_flags, describe_note,
-    describe_attr_tag_arm, describe_symbol_other
+    describe_attr_tag_arm, describe_attr_tag_riscv, describe_symbol_other
     )
 from elftools.elf.constants import E_FLAGS
 from elftools.elf.constants import E_FLAGS_MASKS
@@ -96,6 +96,13 @@ def _get_cu_base(cu):
     else:
         raise ValueError("Can't find the base IP (low_pc) for a CU")
 
+# Matcher for all control characters, for transforming them into "^X" form when
+# formatting symbol names for display.
+_CONTROL_CHAR_RE = re.compile(r'[\x01-\x1f]')
+
+def _format_symbol_name(s):
+    return _CONTROL_CHAR_RE.sub(lambda match: '^' + chr(0x40 + ord(match[0])), s)
+
 class ReadElf(object):
     """ display_* methods are used to emit output into the output stream
     """
@@ -121,7 +128,7 @@ class ReadElf(object):
         """
         self._emitline('ELF Header:')
         self._emit('  Magic:   ')
-        self._emit(' '.join('%2.2x' % byte2int(b)
+        self._emit(' '.join('%2.2x' % b
                    for b in self.elffile.e_ident_raw))
         self._emitline('      ')
         header = self.elffile.header
@@ -244,6 +251,34 @@ class ReadElf(object):
             if (flags & E_FLAGS.EF_MIPS_ARCH) == E_FLAGS.EF_MIPS_ARCH_64:
                 description += ", mips64"
 
+        elif self.elffile['e_machine'] == "EM_RISCV":
+            if flags & E_FLAGS.EF_RISCV_RVC:
+                description += ", RVC"
+            if (flags & E_FLAGS.EF_RISCV_RVE):
+                description += ", RVE"
+            if (flags & E_FLAGS.EF_RISCV_TSO):
+                description += ", TSO"
+            if (flags & E_FLAGS.EF_RISCV_FLOAT_ABI) == E_FLAGS.EF_RISCV_FLOAT_ABI_SOFT:
+                description += ", soft-float ABI"
+            if (flags & E_FLAGS.EF_RISCV_FLOAT_ABI) == E_FLAGS.EF_RISCV_FLOAT_ABI_SINGLE:
+                description += ", single-float ABI"
+            if (flags & E_FLAGS.EF_RISCV_FLOAT_ABI) == E_FLAGS.EF_RISCV_FLOAT_ABI_DOUBLE:
+                description += ", double-float ABI"
+            if (flags & E_FLAGS.EF_RISCV_FLOAT_ABI) == E_FLAGS.EF_RISCV_FLOAT_ABI_QUAD:
+                description += ", quad-float ABI"
+
+        elif self.elffile['e_machine'] == "EM_LOONGARCH":
+            if (flags & E_FLAGS.EF_LOONGARCH_ABI_MODIFIER_MASK) == E_FLAGS.EF_LOONGARCH_ABI_SOFT_FLOAT:
+                description += ", SOFT-FLOAT"
+            if (flags & E_FLAGS.EF_LOONGARCH_ABI_MODIFIER_MASK) == E_FLAGS.EF_LOONGARCH_ABI_SINGLE_FLOAT:
+                description += ", SINGLE-FLOAT"
+            if (flags & E_FLAGS.EF_LOONGARCH_ABI_MODIFIER_MASK) == E_FLAGS.EF_LOONGARCH_ABI_DOUBLE_FLOAT:
+                description += ", DOUBLE-FLOAT"
+            if (flags & E_FLAGS.EF_LOONGARCH_OBJABI_MASK) == E_FLAGS.EF_LOONGARCH_OBJABI_V0:
+                description += ", OBJ-v0"
+            if (flags & E_FLAGS.EF_LOONGARCH_OBJABI_MASK) == E_FLAGS.EF_LOONGARCH_OBJABI_V1:
+                description += ", OBJ-v1"
+
         return description
 
     def display_program_headers(self, show_heading=True):
@@ -468,7 +503,7 @@ class ReadElf(object):
                     describe_symbol_shndx(self._get_symbol_shndx(symbol,
                                                                  nsym,
                                                                  section_index)),
-                    symbol_name,
+                    _format_symbol_name(symbol_name),
                     version_info))
 
     def display_dynamic_tags(self):
@@ -605,7 +640,7 @@ class ReadElf(object):
                         self._format_hex(
                             symbol['st_value'],
                             fullhex=True, lead0x=False),
-                        symbol_name))
+                        _format_symbol_name(symbol_name)))
                     if section.is_RELA():
                         self._emit(' %s %x' % (
                             '+' if rel['r_addend'] >= 0 else '-',
@@ -769,6 +804,8 @@ class ReadElf(object):
         """
         if self.elffile['e_machine'] == 'EM_ARM':
             self._display_arch_specific_arm()
+        elif self.elffile['e_machine'] == 'EM_RISCV':
+            self._display_arch_specific_riscv()
 
     def display_hex_dump(self, section_spec):
         """ Display a hex dump of a section. section_spec is either a section
@@ -800,7 +837,7 @@ class ReadElf(object):
             self._emit('  %s ' % self._format_hex(addr, fieldsize=8))
             for i in range(16):
                 if i < linebytes:
-                    self._emit('%2.2x' % byte2int(data[dataptr + i]))
+                    self._emit('%2.2x' % data[dataptr + i])
                 else:
                     self._emit('  ')
                 if i % 4 == 3:
@@ -808,7 +845,7 @@ class ReadElf(object):
 
             for i in range(linebytes):
                 c = data[dataptr + i : dataptr + i + 1]
-                if byte2int(c[0]) >= 32 and byte2int(c[0]) < 0x7f:
+                if c[0] >= 32 and c[0] < 0x7f:
                     self._emit(bytes2str(c))
                 else:
                     self._emit(bytes2str(b'.'))
@@ -843,14 +880,14 @@ class ReadElf(object):
 
         while dataptr < len(data):
             while ( dataptr < len(data) and
-                    not (32 <= byte2int(data[dataptr]) <= 127)):
+                    not (32 <= data[dataptr] <= 127)):
                 dataptr += 1
 
             if dataptr >= len(data):
                 break
 
             endptr = dataptr
-            while endptr < len(data) and byte2int(data[endptr]) != 0:
+            while endptr < len(data) and data[endptr] != 0:
                 endptr += 1
 
             found = True
@@ -920,7 +957,8 @@ class ReadElf(object):
                 lead0x = False
             else:
                 lead0x = True
-                fieldsize -= 2
+                if fieldsize is not None:
+                    fieldsize -= 2
 
         s = '0x' if lead0x else ''
         if fullhex:
@@ -951,7 +989,7 @@ class ReadElf(object):
             self._format_hex(
                 version_section['sh_addr'], fieldsize=16, lead0x=lead0x),
             self._format_hex(
-                version_section['sh_offset'], fieldsize=6, lead0x=True),
+                version_section['sh_offset'], fieldsize=8, lead0x=True),
             version_section['sh_link'],
                 self.elffile.get_section(version_section['sh_link']).name
             )
@@ -1088,17 +1126,28 @@ class ReadElf(object):
 
         for cu in self._dwarfinfo.iter_CUs():
             self._emitline('  Compilation Unit @ offset %s:' %
-                self._format_hex(cu.cu_offset))
+                self._format_hex(cu.cu_offset, alternate=True))
             self._emitline('   Length:        %s (%s)' % (
                 self._format_hex(cu['unit_length']),
                 '%s-bit' % cu.dwarf_format()))
             self._emitline('   Version:       %s' % cu['version'])
-            if cu.header.get("unit_type", False):
-                ut = next((key for key, value in ENUM_DW_UT.items() if value == cu.header.unit_type), '?')
-                self._emitline('   Unit Type:     %s (%d)' % (ut, cu.header.unit_type))
-            self._emitline('   Abbrev Offset: %s' % (
-                self._format_hex(cu['debug_abbrev_offset']))),
-            self._emitline('   Pointer Size:  %s' % cu['address_size'])
+            if cu['version'] >= 5:
+                if cu.header.get("unit_type", ''):
+                    unit_type = cu.header.unit_type
+                    self._emitline('   Unit Type:     %s (%d)' % (
+                        unit_type, ENUM_DW_UT.get(cu.header.unit_type, 0)))
+                    self._emitline('   Abbrev Offset: %s' % (
+                        self._format_hex(cu['debug_abbrev_offset'], alternate=True)))
+                    self._emitline('   Pointer Size:  %s' % cu['address_size'])
+                    if unit_type in ('DW_UT_skeleton', 'DW_UT_split_compile'):
+                        self._emitline('   Dwo id:        %s' % cu['dwo_id'])
+                    elif unit_type in ('DW_UT_type', 'DW_UT_split_type'):
+                        self._emitline('   Signature:     0x%x' % cu['type_signature'])
+                        self._emitline('   Type Offset:   0x%x' % cu['type_offset'])
+            else:
+                self._emitline('   Abbrev Offset: %s' % (
+                    self._format_hex(cu['debug_abbrev_offset'], alternate=True))),
+                self._emitline('   Pointer Size:  %s' % cu['address_size'])
 
             # The nesting depth of each DIE within the tree of DIEs must be
             # displayed. To implement this, a counter is incremented each time
@@ -1120,7 +1169,7 @@ class ReadElf(object):
                     die_depth -= 1
                     continue
 
-                for attr in itervalues(die.attributes):
+                for attr in die.attributes.values():
                     name = attr.name
                     # Unknown attribute values are passed-through as integers
                     if isinstance(name, int):
@@ -1152,9 +1201,16 @@ class ReadElf(object):
             return
         self._emitline('Contents of the %s section:' % self._dwarfinfo.debug_line_sec.name)
         self._emitline()
+        lineprogram_list = []
 
         for cu in self._dwarfinfo.iter_CUs():
+            # Avoid dumping same lineprogram multiple times
             lineprogram = self._dwarfinfo.line_program_for_CU(cu)
+
+            if lineprogram in lineprogram_list:
+                continue 
+
+            lineprogram_list.append(lineprogram)
             ver5 = lineprogram.header.version >= 5
 
             cu_filename = bytes2str(lineprogram['file_entry'][0].name)
@@ -1403,24 +1459,20 @@ class ReadElf(object):
 
             # Look at the registers the decoded table describes.
             # We build reg_order here to match readelf's order. In particular,
-            # registers are sorted by their number, and the register matching
-            # ra_regnum is always listed last with a special heading.
+            # registers are sorted by their number, so that the register
+            # matching ra_regnum is usually listed last with a special heading.
+            # (LoongArch is a notable exception in that its return register's
+            # DWARF register number is not greater than other GPRs.)
             decoded_table = entry.get_decoded()
-            reg_order = sorted(ifilter(
-                lambda r: r != ra_regnum,
-                decoded_table.reg_order))
+            reg_order = sorted(decoded_table.reg_order)
             if len(decoded_table.reg_order):
-
                 # Headings for the registers
                 for regnum in reg_order:
+                    if regnum == ra_regnum:
+                        self._emit('ra      ')
+                        continue
                     self._emit('%-6s' % describe_reg_name(regnum))
-                self._emitline('ra      ')
-
-                # Now include ra_regnum in reg_order to print its values
-                # similarly to the other registers.
-                reg_order.append(ra_regnum)
-            else:
-                self._emitline()
+            self._emitline()
 
             for line in decoded_table.table:
                 self._emit(self._format_hex(
@@ -1580,33 +1632,54 @@ class ReadElf(object):
             self._dump_debug_rangesection(di, range_lists_sec)
 
     def _dump_debug_rangesection(self, di, range_lists_sec):
-        # In the master branch of binutils, the v5 dump format is way different by now.
-
+        # Last amended to match readelf 2.41
         ver5 = range_lists_sec.version >= 5
         section_name = (di.debug_rnglists_sec if ver5 else di.debug_ranges_sec).name
         addr_size = di.config.default_address_size # In bytes, 4 or 8
         addr_width = addr_size * 2 # In hex digits, 8 or 16
         line_template = "    %%08x %%0%dx %%0%dx %%s" % (addr_width, addr_width)
-        base_template = "    %%08x %%0%dx (base address)" % (addr_width)        
-
-        range_lists = list(range_lists_sec.iter_range_lists())
-        if len(range_lists) == 0:
-            # Present but empty locations section - readelf outputs a message
-            self._emitline("\nSection '%s' has no debugging data." % section_name)
-            return
+        base_template = "    %%08x %%0%dx (base address)" % (addr_width)
 
         # In order to determine the base address of the range
         # We need to know the corresponding CU.
         cu_map = {die.attributes['DW_AT_ranges'].value : cu  # Range list offset => CU
             for cu in di.iter_CUs()
             for die in cu.iter_DIEs()
-            if 'DW_AT_ranges' in die.attributes}
+            if 'DW_AT_ranges' in die.attributes}        
 
-        self._emitline('Contents of the %s section:\n' % section_name)
-        self._emitline('    Offset   Begin    End')
+        if ver5: # Dump by CUs - unsure at this point what does readelf do, ranges dump is buggy in 2.41
+            self._emitline('Contents of the %s section:\n\n\n' % section_name)
+            for cu in range_lists_sec.iter_CUs():
+                self._emitline(' Table at Offset: %s:' % self._format_hex(cu.cu_offset, alternate=True))
+                self._emitline('  Length:          %s' % self._format_hex(cu.unit_length, alternate=True))
+                self._emitline('  DWARF version:   %d' % cu.version)
+                self._emitline('  Address size:    %d' % cu.address_size)
+                self._emitline('  Segment size:    %d' % cu.segment_selector_size)
+                self._emitline('  Offset entries:  %d\n' % cu.offset_count)
+                # Is the offset table dumped too?
+                for (i, range_list) in enumerate(range_lists_sec.iter_CU_range_lists_ex(cu)):
+                    list_offset = range_list[0].entry_offset
+                    range_list = list(range_lists_sec.translate_v5_entry(entry, cu_map[list_offset]) for entry in range_list)
+                    self._emitline('  Offset: %s, Index: %d' % (self._format_hex(list_offset, alternate=True), i))
+                    self._emitline('    Offset   Begin    End')
+                    self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template)
+        else: # Dump by DIE reference offset
+            range_lists = list(range_lists_sec.iter_range_lists())
+            if len(range_lists) == 0:
+                # Present but empty ranges section - readelf outputs a message
+                self._emitline("\nSection '%s' has no debugging data." % section_name)
+                return
 
-        for range_list in range_lists:
-            self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template)
+            self._emitline('Contents of the %s section:\n\n\n' % section_name)
+            self._emitline('    Offset   Begin    End')
+
+            for range_list in range_lists:
+                if len(range_list) == 0: # working around a bogus behavior in readelf 2.41
+                    # No entries means no offset. Dirty hack: peek the stream position
+                    range_list_offset = range_lists_sec.stream.tell() - self._dwarfinfo.config.default_address_size*2
+                    self._emitline('    %08x <End of list>' % (range_list_offset))
+                else:
+                    self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template)
 
     def _dump_rangelist(self, range_list, cu_map, ver5, line_template, base_template):
         # Weird discrepancy in binutils: for DWARFv5 it outputs entry offset,
@@ -1631,22 +1704,30 @@ class ReadElf(object):
         last = range_list[-1]
         self._emitline('    %08x <End of list>' % (last.entry_offset + last.entry_length if ver5 else first.entry_offset))
 
-    def _display_arch_specific_arm(self):
-        """ Display the ARM architecture-specific info contained in the file.
+    def _display_attributes(self, attr_sec, descriptor):
+        """ Display the attributes contained in the section.
         """
-        attr_sec = self.elffile.get_section_by_name('.ARM.attributes')
-
         for s in attr_sec.iter_subsections():
             self._emitline("Attribute Section: %s" % s.header['vendor_name'])
             for ss in s.iter_subsubsections():
                 h_val = "" if ss.header.extra is None else " ".join("%d" % x for x in ss.header.extra)
-                self._emitline(describe_attr_tag_arm(ss.header.tag, h_val, None))
+                self._emitline(descriptor(ss.header.tag, h_val, None))
 
                 for attr in ss.iter_attributes():
                     self._emit('  ')
-                    self._emitline(describe_attr_tag_arm(attr.tag,
-                                                         attr.value,
-                                                         attr.extra))
+                    self._emitline(descriptor(attr.tag, attr.value, attr.extra))
+
+    def _display_arch_specific_arm(self):
+        """ Display the ARM architecture-specific info contained in the file.
+        """
+        attr_sec = self.elffile.get_section_by_name('.ARM.attributes')
+        self._display_attributes(attr_sec, describe_attr_tag_arm)
+
+    def _display_arch_specific_riscv(self):
+        """ Display the RISC-V architecture-specific info contained in the file.
+        """
+        attr_sec = self.elffile.get_section_by_name('.riscv.attributes')
+        self._display_attributes(attr_sec, describe_attr_tag_riscv)
 
     def _emit(self, s=''):
         """ Emit an object to output
