@@ -53,6 +53,7 @@ ENUM_EI_OSABI = dict(
     ELFOSABI_SORTIX=53,
     ELFOSABI_ARM_AEABI=64,
     ELFOSABI_ARM=97,
+    ELFOSABI_CELL_LV2=102,
     ELFOSABI_STANDALONE=255,
     _default_=Pass,
 )
@@ -254,6 +255,10 @@ ENUM_E_MACHINE = dict(
     EM_MOXIE         = 223, # Moxie processor family
     EM_AMDGPU        = 224, # AMD GPU architecture
     EM_RISCV         = 243, # RISC-V
+    EM_BPF           = 247, # Linux BPF - in-kernel virtual machine
+    EM_CSKY          = 252, # C-SKY
+    EM_LOONGARCH     = 258, # LoongArch
+    EM_FRV           = 0x5441, # Fujitsu FR-V
     # Reservations
     # reserved  11-14   Reserved for future use
     # reserved  16      Reserved for future use
@@ -291,7 +296,8 @@ ENUM_SH_TYPE_BASE = dict(
     SHT_PREINIT_ARRAY=16,
     SHT_GROUP=17,
     SHT_SYMTAB_SHNDX=18,
-    SHT_NUM=19,
+    SHT_RELR=19,
+    SHT_NUM=20,
     SHT_LOOS=0x60000000,
     SHT_GNU_ATTRIBUTES=0x6ffffff5,
     SHT_GNU_HASH=0x6ffffff6,
@@ -324,6 +330,10 @@ ENUM_SH_TYPE_ARM = merge_dicts(
             SHT_ARM_ATTRIBUTES=0x70000003,
             SHT_ARM_DEBUGOVERLAY=0x70000004))
 
+ENUM_SH_TYPE_RISCV = merge_dicts(
+        ENUM_SH_TYPE_BASE,
+        dict(SHT_RISCV_ATTRIBUTES=0x70000003))
+
 ENUM_SH_TYPE_MIPS = merge_dicts(
         ENUM_SH_TYPE_BASE,
         dict(
@@ -361,7 +371,8 @@ ENUM_SH_TYPE_MIPS = merge_dicts(
             SHT_MIPS_WHIRL=0x70000026,
             SHT_MIPS_EH_REGION=0x70000027,
             SHT_MIPS_XLATE_OLD=0x70000028,
-            SHT_MIPS_PDR_EXCEPTION=0x70000029))
+            SHT_MIPS_PDR_EXCEPTION=0x70000029,
+            SHT_MIPS_ABIFLAGS=0x7000002a))
 
 ENUM_ELFCOMPRESS_TYPE = dict(
     ELFCOMPRESS_ZLIB=1,
@@ -396,6 +407,7 @@ ENUM_P_TYPE_BASE = dict(
     PT_GNU_EH_FRAME=0x6474e550,
     PT_GNU_STACK=0x6474e551,
     PT_GNU_RELRO=0x6474e552,
+    PT_GNU_PROPERTY=0x6474e553,
     _default_=Pass,
 )
 
@@ -415,6 +427,10 @@ ENUM_P_TYPE_MIPS = merge_dicts(
         ENUM_P_TYPE_BASE,
         dict(PT_MIPS_ABIFLAGS=0x70000003))
 
+ENUM_P_TYPE_RISCV = merge_dicts(
+        ENUM_P_TYPE_BASE,
+        dict(PT_RISCV_ATTRIBUTES=0x70000003))
+
 # st_info bindings in the symbol header
 ENUM_ST_INFO_BIND = dict(
     STB_LOCAL=0,
@@ -459,6 +475,10 @@ ENUM_ST_VISIBILITY = dict(
     _default_=Pass,
 )
 
+ENUM_ST_LOCAL = dict(
+    _default_=Pass,
+)
+
 # st_shndx
 ENUM_ST_SHNDX = dict(
     SHN_UNDEF=0,
@@ -503,8 +523,20 @@ ENUM_D_TAG_COMMON = dict(
     DT_ENCODING=32,
     DT_PREINIT_ARRAY=32,
     DT_PREINIT_ARRAYSZ=33,
-    DT_NUM=34,
+    DT_SYMTAB_SHNDX=34,
+    DT_RELRSZ=35,
+    DT_RELR=36,
+    DT_RELRENT=37,
+    DT_NUM=38,
     DT_LOOS=0x6000000d,
+    DT_ANDROID_REL=0x6000000f,
+    DT_ANDROID_RELSZ=0x60000010,
+    DT_ANDROID_RELA=0x60000011,
+    DT_ANDROID_RELASZ=0x60000012,
+    DT_ANDROID_RELR=0x6fffe000,
+    DT_ANDROID_RELRSZ=0x6fffe001,
+    DT_ANDROID_RELRENT=0x6fffe003,
+    DT_ANDROID_RELRCOUNT=0x6fffe005,
     DT_HIOS=0x6ffff000,
     DT_LOPROC=0x70000000,
     DT_HIPROC=0x7fffffff,
@@ -605,6 +637,45 @@ ENUM_D_TAG.update(ENUM_D_TAG_SOLARIS)
 for k in ENUMMAP_EXTRA_D_TAG_MACHINE:
     ENUM_D_TAG.update(ENUMMAP_EXTRA_D_TAG_MACHINE[k])
 
+ENUM_DT_FLAGS = dict(
+    DF_ORIGIN=0x1,
+    DF_SYMBOLIC=0x2,
+    DF_TEXTREL=0x4,
+    DF_BIND_NOW=0x8,
+    DF_STATIC_TLS=0x10,
+)
+
+ENUM_DT_FLAGS_1 = dict(
+    DF_1_NOW=0x1,
+    DF_1_GLOBAL=0x2,
+    DF_1_GROUP=0x4,
+    DF_1_NODELETE=0x8,
+    DF_1_LOADFLTR=0x10,
+    DF_1_INITFIRST=0x20,
+    DF_1_NOOPEN=0x40,
+    DF_1_ORIGIN=0x80,
+    DF_1_DIRECT=0x100,
+    DF_1_TRANS=0x200,
+    DF_1_INTERPOSE=0x400,
+    DF_1_NODEFLIB=0x800,
+    DF_1_NODUMP=0x1000,
+    DF_1_CONFALT=0x2000,
+    DF_1_ENDFILTEE=0x4000,
+    DF_1_DISPRELDNE=0x8000,
+    DF_1_DISPRELPND=0x10000,
+    DF_1_NODIRECT=0x20000,
+    DF_1_IGNMULDEF=0x40000,
+    DF_1_NOKSYMS=0x80000,
+    DF_1_NOHDR=0x100000,
+    DF_1_EDITED=0x200000,
+    DF_1_NORELOC=0x400000,
+    DF_1_SYMINTPOSE=0x800000,
+    DF_1_GLOBAUDIT=0x1000000,
+    DF_1_SINGLETON=0x2000000,
+    DF_1_STUB=0x4000000,
+    DF_1_PIE=0x8000000,
+)
+
 ENUM_RELOC_TYPE_MIPS = dict(
     R_MIPS_NONE=0,
     R_MIPS_16=1,
@@ -744,11 +815,125 @@ ENUM_RELOC_TYPE_x64 = dict(
     R_X86_64_TLSDESC_CALL=35,
     R_X86_64_TLSDESC=36,
     R_X86_64_IRELATIVE=37,
+    R_X86_64_REX_GOTPCRELX=42,
     R_X86_64_GNU_VTINHERIT=250,
     R_X86_64_GNU_VTENTRY=251,
     _default_=Pass,
 )
 
+ENUM_RELOC_TYPE_BPF = dict(
+    R_BPF_NONE=0,
+    R_BPF_64_64=1,
+    R_BPF_64_ABS64=2,
+    R_BPF_64_ABS32=3,
+    R_BPF_64_NODYLD32=4,
+    R_BPF_64_32=10,
+    _default_=Pass,
+)
+
+# https://github.com/loongson/la-abi-specs/blob/release/laelf.adoc
+ENUM_RELOC_TYPE_LOONGARCH = dict(
+    R_LARCH_NONE=0,
+    R_LARCH_32=1,
+    R_LARCH_64=2,
+    R_LARCH_RELATIVE=3,
+    R_LARCH_COPY=4,
+    R_LARCH_JUMP_SLOT=5,
+    R_LARCH_TLS_DTPMOD32=6,
+    R_LARCH_TLS_DTPMOD64=7,
+    R_LARCH_TLS_DTPREL32=8,
+    R_LARCH_TLS_DTPREL64=9,
+    R_LARCH_TLS_TPREL32=10,
+    R_LARCH_TLS_TPREL64=11,
+    R_LARCH_IRELATIVE=12,
+    R_LARCH_MARK_LA=20,
+    R_LARCH_MARK_PCREL=21,
+    R_LARCH_SOP_PUSH_PCREL=22,
+    R_LARCH_SOP_PUSH_ABSOLUTE=23,
+    R_LARCH_SOP_PUSH_DUP=24,
+    R_LARCH_SOP_PUSH_GPREL=25,
+    R_LARCH_SOP_PUSH_TLS_TPREL=26,
+    R_LARCH_SOP_PUSH_TLS_GOT=27,
+    R_LARCH_SOP_PUSH_TLS_GD=28,
+    R_LARCH_SOP_PUSH_PLT_PCREL=29,
+    R_LARCH_SOP_ASSERT=30,
+    R_LARCH_SOP_NOT=31,
+    R_LARCH_SOP_SUB=32,
+    R_LARCH_SOP_SL=33,
+    R_LARCH_SOP_SR=34,
+    R_LARCH_SOP_ADD=35,
+    R_LARCH_SOP_AND=36,
+    R_LARCH_SOP_IF_ELSE=37,
+    R_LARCH_SOP_POP_32_S_10_5=38,
+    R_LARCH_SOP_POP_32_U_10_12=39,
+    R_LARCH_SOP_POP_32_S_10_12=40,
+    R_LARCH_SOP_POP_32_S_10_16=41,
+    R_LARCH_SOP_POP_32_S_10_16_S2=42,
+    R_LARCH_SOP_POP_32_S_5_20=43,
+    R_LARCH_SOP_POP_32_S_0_5_10_16_S2=44,
+    R_LARCH_SOP_POP_32_S_0_10_10_16_S2=45,
+    R_LARCH_SOP_POP_32_U=46,
+    R_LARCH_ADD8=47,
+    R_LARCH_ADD16=48,
+    R_LARCH_ADD24=49,
+    R_LARCH_ADD32=50,
+    R_LARCH_ADD64=51,
+    R_LARCH_SUB8=52,
+    R_LARCH_SUB16=53,
+    R_LARCH_SUB24=54,
+    R_LARCH_SUB32=55,
+    R_LARCH_SUB64=56,
+    R_LARCH_GNU_VTINHERIT=57,
+    R_LARCH_GNU_VTENTRY=58,
+    R_LARCH_B16=64,
+    R_LARCH_B21=65,
+    R_LARCH_B26=66,
+    R_LARCH_ABS_HI20=67,
+    R_LARCH_ABS_LO12=68,
+    R_LARCH_ABS64_LO20=69,
+    R_LARCH_ABS64_HI12=70,
+    R_LARCH_PCALA_HI20=71,
+    R_LARCH_PCALA_LO12=72,
+    R_LARCH_PCALA64_LO20=73,
+    R_LARCH_PCALA64_HI12=74,
+    R_LARCH_GOT_PC_HI20=75,
+    R_LARCH_GOT_PC_LO12=76,
+    R_LARCH_GOT64_PC_LO20=77,
+    R_LARCH_GOT64_PC_HI12=78,
+    R_LARCH_GOT_HI20=79,
+    R_LARCH_GOT_LO12=80,
+    R_LARCH_GOT64_LO20=81,
+    R_LARCH_GOT64_HI12=82,
+    R_LARCH_TLS_LE_HI20=83,
+    R_LARCH_TLS_LE_LO12=84,
+    R_LARCH_TLS_LE64_LO20=85,
+    R_LARCH_TLS_LE64_HI12=86,
+    R_LARCH_TLS_IE_PC_HI20=87,
+    R_LARCH_TLS_IE_PC_LO12=88,
+    R_LARCH_TLS_IE64_PC_LO20=89,
+    R_LARCH_TLS_IE64_PC_HI12=90,
+    R_LARCH_TLS_IE_HI20=91,
+    R_LARCH_TLS_IE_LO12=92,
+    R_LARCH_TLS_IE64_LO20=93,
+    R_LARCH_TLS_IE64_HI12=94,
+    R_LARCH_TLS_LD_PC_HI20=95,
+    R_LARCH_TLS_LD_HI20=96,
+    R_LARCH_TLS_GD_PC_HI20=97,
+    R_LARCH_TLS_GD_HI20=98,
+    R_LARCH_32_PCREL=99,
+    R_LARCH_RELAX=100,
+    R_LARCH_DELETE=101,
+    R_LARCH_ALIGN=102,
+    R_LARCH_PCREL20_S2=103,
+    R_LARCH_CFA=104,
+    R_LARCH_ADD6=105,
+    R_LARCH_SUB6=106,
+    R_LARCH_ADD_ULEB128=107,
+    R_LARCH_SUB_ULEB128=108,
+    R_LARCH_64_PCREL=109,
+    _default_=Pass,
+)
+
 # Sunw Syminfo Bound To special values
 ENUM_SUNW_SYMINFO_BOUNDTO = dict(
     SYMINFO_BT_SELF=0xffff,
@@ -782,6 +967,7 @@ ENUM_NOTE_N_TYPE = dict(
     NT_GNU_HWCAP=2,
     NT_GNU_BUILD_ID=3,
     NT_GNU_GOLD_VERSION=4,
+    NT_GNU_PROPERTY_TYPE_0=5,
     _default_=Pass,
 )
 
@@ -808,6 +994,25 @@ ENUM_NOTE_ABI_TAG_OS = dict(
     _default_=Pass,
 )
 
+# Values in GNU .note.gnu.property notes (n_type=='NT_GNU_PROPERTY_TYPE_0')
+ENUM_NOTE_GNU_PROPERTY_TYPE = dict(
+    GNU_PROPERTY_STACK_SIZE=1,
+    GNU_PROPERTY_NO_COPY_ON_PROTECTED=2,
+    GNU_PROPERTY_X86_FEATURE_1_AND=0xc0000002,
+    GNU_PROPERTY_X86_ISA_1_NEEDED=0xc0008002,
+    GNU_PROPERTY_X86_FEATURE_2_USED=0xc0010001,
+    GNU_PROPERTY_X86_ISA_1_USED=0xc0010002,
+    _default_=Pass,
+)
+
+ENUM_GNU_PROPERTY_X86_FEATURE_1_FLAGS = dict(
+    GNU_PROPERTY_X86_FEATURE_1_IBT=1,
+    GNU_PROPERTY_X86_FEATURE_1_SHSTK=2,
+    GNU_PROPERTY_X86_FEATURE_1_LAM_U48=4,
+    GNU_PROPERTY_X86_FEATURE_1_LAM_U57=8,
+    _default_=Pass
+)
+
 ENUM_RELOC_TYPE_ARM = dict(
     R_ARM_NONE=0,
     R_ARM_PC24=1,
@@ -1089,6 +1294,7 @@ ENUM_ATTR_TAG_ARM = dict(
     TAG_ABI_FP_16BIT_FORMAT=38,
     TAG_MPEXTENSION_USE=42,
     TAG_DIV_USE=44,
+    TAG_DSP_EXTENSION = 46,
     TAG_NODEFAULTS=64,
     TAG_ALSO_COMPATIBLE_WITH=65,
     TAG_T2EE_USE=66,
@@ -1096,3 +1302,139 @@ ENUM_ATTR_TAG_ARM = dict(
     TAG_VIRTUALIZATION_USE=68,
     TAG_MPEXTENSION_USE_OLD=70,
 )
+
+ENUM_ATTR_TAG_RISCV = dict(
+    TAG_FILE=1,
+    TAG_SECTION=2,
+    TAG_SYMBOL=3,
+    TAG_STACK_ALIGN=4,
+    TAG_ARCH=5,
+    TAG_UNALIGNED_ACCESS=6,
+)
+
+# https://openpowerfoundation.org/wp-content/uploads/2016/03/ABI64BitOpenPOWERv1.1_16July2015_pub4.pdf
+# See 3.5.3 Relocation Types Table.
+ENUM_RELOC_TYPE_PPC64 = dict(
+    R_PPC64_NONE=0,
+    R_PPC64_ADDR32=1,
+    R_PPC64_ADDR24=2,
+    R_PPC64_ADDR16=3,
+    R_PPC64_ADDR16_LO=4,
+    R_PPC64_ADDR16_HI=5,
+    R_PPC64_ADDR16_HA=6,
+    R_PPC64_ADDR14=7,
+    R_PPC64_ADDR14_BRTAKEN=8,
+    R_PPC64_ADDR14_BRNTAKEN=9,
+    R_PPC64_REL24=10,
+    R_PPC64_REL14=11,
+    R_PPC64_REL14_BRTAKEN=12,
+    R_PPC64_REL14_BRNTAKEN=13,
+    R_PPC64_GOT16=14,
+    R_PPC64_GOT16_LO=15,
+    R_PPC64_GOT16_HI=16,
+    R_PPC64_GOT16_HA=17,
+    R_PPC64_COPY=19,
+    R_PPC64_GLOB_DAT=20,
+    R_PPC64_JMP_SLOT=21,
+    R_PPC64_RELATIVE=22,
+    R_PPC64_UADDR32=24,
+    R_PPC64_UADDR16=25,
+    R_PPC64_REL32=26,
+    R_PPC64_PLT32=27,
+    R_PPC64_PLTREL32=28,
+    R_PPC64_PLT16_LO=29,
+    R_PPC64_PLT16_HI=30,
+    R_PPC64_PLT16_HA=31,
+    R_PPC64_SECTOFF=33,
+    R_PPC64_SECTOFF_LO=34,
+    R_PPC64_SECTOFF_HI=35,
+    R_PPC64_SECTOFF_HA=36,
+    R_PPC64_ADDR30=37,
+    R_PPC64_ADDR64=38,
+    R_PPC64_ADDR16_HIGHER=39,
+    R_PPC64_ADDR16_HIGHERA=40,
+    R_PPC64_ADDR16_HIGHEST=41,
+    R_PPC64_ADDR16_HIGHESTA=42,
+    R_PPC64_UADDR64=43,
+    R_PPC64_REL64=44,
+    R_PPC64_PLT64=45,
+    R_PPC64_PLTREL64=46,
+    R_PPC64_TOC16=47,
+    R_PPC64_TOC16_LO=48,
+    R_PPC64_TOC16_HI=49,
+    R_PPC64_TOC16_HA=50,
+    R_PPC64_TOC=51,
+    R_PPC64_PLTGOT16=52,
+    R_PPC64_PLTGOT16_LO=53,
+    R_PPC64_PLTGOT16_HI=54,
+    R_PPC64_PLTGOT16_HA=55,
+    R_PPC64_ADDR16_DS=56,
+    R_PPC64_ADDR16_LO_DS=57,
+    R_PPC64_GOT16_DS=58,
+    R_PPC64_GOT16_LO_DS=59,
+    R_PPC64_PLT16_LO_DS=60,
+    R_PPC64_SECTOFF_DS=61,
+    R_PPC64_SECTOFF_LO_DS=62,
+    R_PPC64_TOC16_DS=63,
+    R_PPC64_TOC16_LO_DS=64,
+    R_PPC64_PLTGOT16_DS=65,
+    R_PPC64_PLTGOT16_LO_DS=66,
+    R_PPC64_TLS=67,
+    R_PPC64_DTPMOD64=68,
+    R_PPC64_TPREL16=69,
+    R_PPC64_TPREL16_LO=70,
+    R_PPC64_TPREL16_HI=71,
+    R_PPC64_TPREL16_HA=72,
+    R_PPC64_TPREL64=73,
+    R_PPC64_DTPREL16=74,
+    R_PPC64_DTPREL16_LO=75,
+    R_PPC64_DTPREL16_HI=76,
+    R_PPC64_DTPREL16_HA=77,
+    R_PPC64_DTPREL64=78,
+    R_PPC64_GOT_TLSGD16=79,
+    R_PPC64_GOT_TLSGD16_LO=80,
+    R_PPC64_GOT_TLSGD16_HI=81,
+    R_PPC64_GOT_TLSGD16_HA=82,
+    R_PPC64_GOT_TLSLD16=83,
+    R_PPC64_GOT_TLSLD16_LO=84,
+    R_PPC64_GOT_TLSLD16_HI=85,
+    R_PPC64_GOT_TLSLD16_HA=86,
+    R_PPC64_GOT_TPREL16_DS=87,
+    R_PPC64_GOT_TPREL16_LO_DS=88,
+    R_PPC64_GOT_TPREL16_HI=89,
+    R_PPC64_GOT_TPREL16_HA=90,
+    R_PPC64_GOT_DTPREL16_DS=91,
+    R_PPC64_GOT_DTPREL16_LO_DS=92,
+    R_PPC64_GOT_DTPREL16_HI=93,
+    R_PPC64_GOT_DTPREL16_HA=94,
+    R_PPC64_TPREL16_DS=95,
+    R_PPC64_TPREL16_LO_DS=96,
+    R_PPC64_TPREL16_HIGHER=97,
+    R_PPC64_TPREL16_HIGHERA=98,
+    R_PPC64_TPREL16_HIGHEST=99,
+    R_PPC64_TPREL16_HIGHESTA=100,
+    R_PPC64_DTPREL16_DS=101,
+    R_PPC64_DTPREL16_LO_DS=102,
+    R_PPC64_DTPREL16_HIGHER=103,
+    R_PPC64_DTPREL16_HIGHERA=104,
+    R_PPC64_DTPREL16_HIGHEST=105,
+    R_PPC64_DTPREL16_HIGHESTA=106,
+    R_PPC64_TLSGD=107,
+    R_PPC64_TLSLD=108,
+    R_PPC64_TOCSAVE=109,
+    R_PPC64_ADDR16_HIGH=110,
+    R_PPC64_ADDR16_HIGHA=111,
+    R_PPC64_TPREL16_HIGH=112,
+    R_PPC64_TPREL16_HIGHA=113,
+    R_PPC64_DTPREL16_HIGH=114,
+    R_PPC64_DTPREL16_HIGHA=115,
+    R_PPC64_REL24_NOTOC=116,
+    R_PPC64_ADDR64_LOCAL=117,
+    R_PPC64_IRELATIVE=248,
+    R_PPC64_REL16=249,
+    R_PPC64_REL16_LO=250,
+    R_PPC64_REL16_HI=251,
+    R_PPC64_REL16_HA=252,
+    R_PPC64_GNU_VTINHERIT=253,
+    R_PPC64_GNU_VTENTRY=254,
+)
